@@ -25,15 +25,22 @@
 #include "config.h"
 #endif
 
+#define FF_API_CODEC_ID 1
+
 #ifdef HAVE_FFMPEG_AVUTIL_H
 #  include <avcodec.h>
 #else
 #  include <libavcodec/avcodec.h>
+#  include <libavutil/mem.h>
 #endif
 
 typedef struct ff_codec_s {
   uint32_t          type;
+#if defined LIBAVCODEC_VERSION_INT && LIBAVCODEC_VERSION_INT >= ((54<<16)|(25<<8))
+  enum AVCodecID    id;
+#else
   enum CodecID      id;
+#endif
   const char       *name;
 } ff_codec_t;
 
