@@ -123,10 +123,10 @@
 #define LOG
 */
 
-#include "xine_internal.h"
-#include "xineutils.h"
-#include "compat.h"
-#include "demux.h"
+#include <xine/xine_internal.h>
+#include <xine/xineutils.h>
+#include <xine/compat.h>
+#include <xine/demux.h>
 #include "bswap.h"
 #include "group_games.h"
 
@@ -515,11 +515,6 @@ static int demux_str_seek (demux_plugin_t *this_gen, off_t start_pos, int start_
   return this->status;
 }
 
-static void demux_str_dispose (demux_plugin_t *this) {
-
-  free(this);
-}
-
 static int demux_str_get_status (demux_plugin_t *this_gen) {
   demux_str_t *this = (demux_str_t *) this_gen;
 
@@ -559,7 +554,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   this->demux_plugin.send_headers      = demux_str_send_headers;
   this->demux_plugin.send_chunk        = demux_str_send_chunk;
   this->demux_plugin.seek              = demux_str_seek;
-  this->demux_plugin.dispose           = demux_str_dispose;
+  this->demux_plugin.dispose           = default_demux_plugin_dispose;
   this->demux_plugin.get_status        = demux_str_get_status;
   this->demux_plugin.get_stream_length = demux_str_get_stream_length;
   this->demux_plugin.get_capabilities  = demux_str_get_capabilities;
@@ -570,19 +565,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   switch (stream->content_detection_method) {
 
-  case METHOD_BY_EXTENSION: {
-    const char *extensions, *mrl;
-
-    mrl = input->get_mrl (input);
-    extensions = class_gen->get_extensions (class_gen);
-
-    if (!_x_demux_check_extension (mrl, extensions)) {
-      free (this);
-      return NULL;
-    }
-  }
-  /* falling through is intended */
-
+  case METHOD_BY_MRL:
   case METHOD_BY_CONTENT:
   case METHOD_EXPLICIT:
 
@@ -600,39 +583,17 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   return &this->demux_plugin;
 }
 
-static const char *get_description (demux_class_t *this_gen) {
-  return "Sony Playstation STR file demux plugin";
-}
-
-static const char *get_identifier (demux_class_t *this_gen) {
-  return "PSX STR";
-}
-
-static const char *get_extensions (demux_class_t *this_gen) {
-  /* also .mov, but we don't want to hijack that extension */
-  return "str iki ik2 dps dat xa xa1 xa2 xas xap";
-}
-
-static const char *get_mimetypes (demux_class_t *this_gen) {
-  return NULL;
-}
-
-static void class_dispose (demux_class_t *this_gen) {
-  demux_str_class_t *this = (demux_str_class_t *) this_gen;
-  free (this);
-}
-
 void *demux_str_init_plugin (xine_t *xine, void *data) {
   demux_str_class_t     *this;
 
   this = calloc(1, sizeof(demux_str_class_t));
 
   this->demux_class.open_plugin     = open_plugin;
-  this->demux_class.get_description = get_description;
-  this->demux_class.get_identifier  = get_identifier;
-  this->demux_class.get_mimetypes   = get_mimetypes;
-  this->demux_class.get_extensions  = get_extensions;
-  this->demux_class.dispose         = class_dispose;
+  this->demux_class.description     = N_("Sony Playstation STR file demux plugin");
+  this->demux_class.identifier      = "PSX STR";
+  this->demux_class.mimetypes       = NULL;
+  this->demux_class.extensions      = "str iki ik2 dps dat xa xa1 xa2 xas xap";
+  this->demux_class.dispose         = default_demux_class_dispose;
 
   return this;
 }
