@@ -785,8 +785,11 @@ static void decode_picture( vdpau_mpeg12_decoder_t *vd, uint8_t end_of_sequence
   }
 
   int still_image = (end_of_sequence) ? VO_STILL_IMAGE : 0;
+  /* no sequence display extension parser yet so at least enable autoselection */
+  int color_matrix = 0;
+  VO_SET_FLAGS_CM (4, color_matrix);
   vo_frame_t *img = vd->stream->video_out->get_frame( vd->stream->video_out, seq->coded_width, seq->coded_height,
-                                                      seq->ratio, XINE_IMGFMT_VDPAU, VO_BOTH_FIELDS | seq->chroma | seq->reset | still_image );
+                                                      seq->ratio, XINE_IMGFMT_VDPAU, VO_BOTH_FIELDS | seq->chroma | seq->reset | still_image | color_matrix );
   seq->reset = 0;                                                      
   vdpau_accel_t *accel = (vdpau_accel_t*)img->accel_data;
   if ( !seq->accel_vdpau )
