@@ -178,8 +178,6 @@ typedef struct {
 
   vdpau_accel_t *accel_vdpau;
 
-  VdpColorStandard  color_standard;
-
   bits_reader_t  br;
 
   int         vdp_runtime_nr;
@@ -196,6 +194,8 @@ typedef struct {
   int         last_non_b_time;
   int         t_frame;
 
+  int         color_matrix;
+
 } sequence_t;
 
 
@@ -280,7 +280,6 @@ static void reset_sequence( sequence_t *sequence, int free_refs )
   sequence->backward_ref = NULL;
   sequence->top_field_first = 0;
   sequence->reset = VO_NEW_SEQUENCE_FLAG;
-  sequence->color_standard = VDP_COLOR_STANDARD_ITUR_BT_709;
 
   sequence->last_time_base = 0;
   sequence->time_base = 0;
@@ -332,6 +331,7 @@ static void visual_object( vdpau_mpeg4_decoder_t *this_gen, uint8_t *buf, int le
 {
   sequence_t *sequence = (sequence_t*)&this_gen->sequence;
   picture_t *picture = (picture_t*)&sequence->picture;
+  int xine_color_matrix = 4; /* undefined, mpeg range */
   bits_reader_set( &sequence->br, buf, len );
 
   if ( read_bits( &sequence->br, 1 ) ) {
@@ -341,16 +341,16 @@ static void visual_object( vdpau_mpeg4_decoder_t *this_gen, uint8_t *buf, int le
   }
   if ( read_bits( &sequence->br, 4 ) == 1 ) {
     if ( read_bits( &sequence->br, 1 ) ) {
-      skip_bits( &sequence->br, 4 );
+      skip_bits (&sequence->br, 3); /* video_format */
+      xine_color_matrix |= read_bits (&sequence->br, 1); /* full range */
       if ( read_bits( &sequence->br, 1 ) ) {
-        if ( read_bits( &sequence->br, 8 ) == 7 ) {
-          lprintf("color_standard: smpte_240M\n");
-          sequence->color_standard = VDP_COLOR_STANDARD_SMPTE_240M;
-        }
-        skip_bits( &sequence->br, 16 );
+        skip_bits (&sequence->br, 16);
+        xine_color_matrix = (xine_color_matrix & 1)
+          | (read_bits (&sequence->br, 8) << 1);  /* matrix_coefficients */
       }
     }
   }
+  VO_SET_FLAGS_CM (xine_color_matrix, sequence->color_matrix);
 }
 
 
@@ -899,7 +899,7 @@ static void decode_picture( vdpau_mpeg4_decoder_t *vd )
     }
   }
 
-  vo_frame_t *img = vd->stream->video_out->get_frame( vd->stream->video_out, seq->coded_width, seq->coded_height, seq->ratio, XINE_IMGFMT_VDPAU, VO_BOTH_FIELDS );
+  vo_frame_t *img = vd->stream->video_out->get_frame( vd->stream->video_out, seq->coded_width, seq->coded_height, seq->ratio, XINE_IMGFMT_VDPAU, VO_BOTH_FIELDS | seq->color_matrix );
   vdpau_accel_t *accel = (vdpau_accel_t*)img->accel_data;
   if ( !seq->accel_vdpau )
     seq->accel_vdpau = accel;
@@ -946,7 +946,6 @@ static void decode_picture( vdpau_mpeg4_decoder_t *vd )
   img->bad_frame = 0;
   if ( seq->video_step > 900 ) /* some buggy streams */
     img->duration = seq->video_step;
-  accel->color_standard = seq->color_standard;
 
   if ( pic->vdp_infos.vop_coding_type < B_FRAME ) {
     if ( pic->vdp_infos.vop_coding_type == I_FRAME && !seq->backward_ref ) {
@@ -1027,7 +1026,6 @@ static void vdpau_mpeg4_decode_data (video_decoder_t *this_gen, buf_element_t *b
  * This function is called when xine needs to flush the system.
  */
 static void vdpau_mpeg4_flush (video_decoder_t *this_gen) {
-  vdpau_mpeg4_decoder_t *this = (vdpau_mpeg4_decoder_t *) this_gen;
 
   lprintf( "vdpau_mpeg4_flush\n" );
 }
