@@ -54,9 +54,6 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 
-#include "xine.h"
-#include <xine/video_out.h>
-
 #include <errno.h>
 
 #include <sys/ipc.h>
@@ -84,6 +81,8 @@
 #define LOG
 */
 
+#include "xine.h"
+#include <xine/video_out.h>
 #include <xine/xine_internal.h>
 #include "yuv2rgb.h"
 #include <xine/xineutils.h>
@@ -623,7 +622,8 @@ static int fb_set_property(vo_driver_t *this_gen, int property, int value)
       this->yuv2rgb_factory->set_csc_levels (this->yuv2rgb_factory,
                                              this->yuv2rgb_brightness,
                                              this->yuv2rgb_contrast,
-                                             this->yuv2rgb_saturation);
+                                             this->yuv2rgb_saturation,
+                                             CM_DEFAULT);
       break;
 
     case VO_PROP_CONTRAST:
@@ -631,7 +631,8 @@ static int fb_set_property(vo_driver_t *this_gen, int property, int value)
       this->yuv2rgb_factory->set_csc_levels (this->yuv2rgb_factory,
                                              this->yuv2rgb_brightness,
                                              this->yuv2rgb_contrast,
-                                             this->yuv2rgb_saturation);
+                                             this->yuv2rgb_saturation,
+                                             CM_DEFAULT);
       break;
 
     case VO_PROP_SATURATION:
@@ -639,7 +640,8 @@ static int fb_set_property(vo_driver_t *this_gen, int property, int value)
       this->yuv2rgb_factory->set_csc_levels (this->yuv2rgb_factory,
                                              this->yuv2rgb_brightness,
                                              this->yuv2rgb_contrast,
-                                             this->yuv2rgb_saturation);
+                                             this->yuv2rgb_saturation,
+                                             CM_DEFAULT);
       break;
 
     default:
@@ -895,7 +897,8 @@ static int setup_yuv2rgb(fb_driver_t *this, config_values_t *config,
   this->yuv2rgb_factory->set_csc_levels (this->yuv2rgb_factory,
                                          this->yuv2rgb_brightness,
                                          this->yuv2rgb_contrast,
-                                         this->yuv2rgb_saturation);
+                                         this->yuv2rgb_saturation,
+                                         CM_DEFAULT);
 
   return 1;
 }
