@@ -31,6 +31,12 @@
  * ported to xcb by Christoph Pfister - Feb 2007
  */
 
+#define LOG_MODULE "video_out_xcbxv"
+#define LOG_VERBOSE
+/*
+#define LOG
+*/
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -51,12 +57,6 @@
 
 #include <xcb/xv.h>
 
-#define LOG_MODULE "video_out_xcbxv"
-#define LOG_VERBOSE
-/*
-#define LOG
-*/
-
 #include "xine.h"
 #include <xine/video_out.h>
 #include <xine/xine_internal.h>
@@ -74,6 +74,8 @@ typedef struct {
   int                max;
   xcb_atom_t         atom;
 
+  int                defer;
+
   cfg_entry_t       *entry;
 
   xv_driver_t       *this;
@@ -99,6 +101,7 @@ typedef struct {
   unsigned int       xv_pitches[3];
   unsigned int       xv_offsets[3];
 
+  int                req_width, req_height;
 } xv_frame_t;
 
 
@@ -144,6 +147,10 @@ struct xv_driver_s {
 
   pthread_mutex_t    main_mutex;
 
+  /* color matrix switching */
+  int                cm_active, cm_state;
+  xcb_atom_t         cm_atom;
+  int                fullrange_mode;
 };
 
 typedef struct {
@@ -153,6 +160,10 @@ typedef struct {
   xine_t              *xine;
 } xv_class_t;
 
+/* import common color matrix stuff */
+#define CM_DRIVER_T xv_driver_t
+#include "color_matrix.c"
+
 VIDEO_DEVICE_XV_DECL_BICUBIC_TYPES;
 VIDEO_DEVICE_XV_DECL_PREFER_TYPES;
 VIDEO_DEVICE_XV_DECL_SYNC_ATOMS;
@@ -360,8 +371,8 @@ static void xv_update_frame_format (vo_driver_t *this_gen,
     width = (width + 7) & ~0x7;
   }
 
-  if ((frame->width != width)
-      || (frame->height != height)
+  if ((frame->req_width != width)
+      || (frame->req_height != height)
       || (frame->format != format)) {
 
     /* printf (LOG_MODULE ": updating frame to %d x %d (ratio=%d, format=%08x)\n",width,height,ratio_code,format); */
@@ -390,13 +401,21 @@ static void xv_update_frame_format (vo_driver_t *this_gen,
       frame->vo_frame.base[2] = frame->image + frame->xv_offsets[1];
     }
 
-    frame->width  = width;
-    frame->height = height;
+    /* allocated frame size may not match requested size */
+    frame->req_width  = width;
+    frame->req_height = height;
+    frame->width  = frame->xv_width;
+    frame->height = frame->xv_height;
     frame->format = format;
 
     pthread_mutex_unlock(&this->main_mutex);
   }
 
+  if (frame->vo_frame.width > frame->width)
+    frame->vo_frame.width  = frame->width;
+  if (frame->vo_frame.height > frame->height)
+    frame->vo_frame.height = frame->height;
+
   frame->ratio = ratio;
 }
 
@@ -539,7 +558,7 @@ static void xv_flush_recent_frames (xv_driver_t *this) {
 
 static int xv_redraw_needed (vo_driver_t *this_gen) {
   xv_driver_t  *this = (xv_driver_t *) this_gen;
-  int           ret  = 0;
+  int           ret  = !this->cm_active;
 
   if( this->cur_frame ) {
 
@@ -569,13 +588,76 @@ static int xv_redraw_needed (vo_driver_t *this_gen) {
   return ret;
 }
 
+static void xv_new_color (xv_driver_t *this, int cm) {
+  int brig = this->props[VO_PROP_BRIGHTNESS].value;
+  int cont = this->props[VO_PROP_CONTRAST].value;
+  int satu = this->props[VO_PROP_SATURATION].value;
+  int cm2, fr = 0, a, b;
+  xcb_atom_t atom;
+
+  if (cm & 1) {
+    /* fullrange emulation. Do not report this via get_property (). */
+    if (this->fullrange_mode == 1) {
+      /* modification routine 1 for TV set style bcs controls 0% - 200% */
+      satu -= this->props[VO_PROP_SATURATION].min;
+      satu  = (satu * (112 * 255) + (127 * 219 / 2)) / (127 * 219);
+      satu += this->props[VO_PROP_SATURATION].min;
+      if (satu > this->props[VO_PROP_SATURATION].max)
+        satu = this->props[VO_PROP_SATURATION].max;
+      cont -= this->props[VO_PROP_CONTRAST].min;
+      cont  = (cont * 219 + 127) / 255;
+      a     = cont * (this->props[VO_PROP_BRIGHTNESS].max - this->props[VO_PROP_BRIGHTNESS].min);
+      cont += this->props[VO_PROP_CONTRAST].min;
+      b     = 256 * (this->props[VO_PROP_CONTRAST].max - this->props[VO_PROP_CONTRAST].min);
+      brig += (16 * a + b / 2) / b;
+      if (brig > this->props[VO_PROP_BRIGHTNESS].max)
+        brig = this->props[VO_PROP_BRIGHTNESS].max;
+      fr = 1;
+    }
+    /* maybe add more routines for non-standard controls later */
+  }
+  pthread_mutex_lock(&this->main_mutex);
+  atom = this->props[VO_PROP_BRIGHTNESS].atom;
+  if (atom != XCB_NONE)
+    xcb_xv_set_port_attribute (this->connection, this->xv_port, atom, brig);
+  atom = this->props[VO_PROP_CONTRAST].atom;
+  if (atom != XCB_NONE)
+    xcb_xv_set_port_attribute (this->connection, this->xv_port, atom, cont);
+  atom = this->props[VO_PROP_SATURATION].atom;
+  if (atom != XCB_NONE)
+    xcb_xv_set_port_attribute (this->connection, this->xv_port, atom, satu);
+  pthread_mutex_unlock(&this->main_mutex);
+
+  /* so far only binary nvidia drivers support this. why not nuveau? */
+  if (this->cm_atom != XCB_NONE) {
+    cm2 = (0xc00c >> cm) & 1;
+    pthread_mutex_lock(&this->main_mutex);
+    xcb_xv_set_port_attribute (this->connection, this->xv_port, this->cm_atom, cm2);
+    pthread_mutex_unlock(&this->main_mutex);
+    cm2 = cm2 ? 2 : 10;
+  } else {
+    cm2 = 10;
+  }
+
+  cm2 |= fr;
+  xprintf (this->xine, XINE_VERBOSITY_LOG, "video_out_xcbxv: %s b %d  c %d  s %d  [%s]\n",
+    fr ? "modified " : "", brig, cont, satu, cm_names[cm2]);
+
+  this->cm_active = cm;
+}
+
 static void xv_display_frame (vo_driver_t *this_gen, vo_frame_t *frame_gen) {
   xv_driver_t  *this  = (xv_driver_t *) this_gen;
   xv_frame_t   *frame = (xv_frame_t *) frame_gen;
+  int cm;
   /*
   printf (LOG_MODULE ": xv_display_frame...\n");
   */
 
+  cm = cm_from_frame (frame_gen);
+  if (cm != this->cm_active)
+    xv_new_color (this, cm);
+
   /*
    * queue frames (deinterlacing)
    * free old frames
@@ -684,6 +766,15 @@ static int xv_set_property (vo_driver_t *this_gen,
 
   if ((property < 0) || (property >= VO_NUM_PROPERTIES)) return 0;
 
+  if (this->props[property].defer == 1) {
+    /* value is out of bound */
+    if((value < this->props[property].min) || (value > this->props[property].max))
+      value = (this->props[property].min + this->props[property].max) >> 1;
+    this->props[property].value = value;
+    this->cm_active = 0;
+    return value;
+  }
+
   if (this->props[property].atom != XCB_NONE) {
     xcb_xv_get_port_attribute_cookie_t get_attribute_cookie;
     xcb_xv_get_port_attribute_reply_t *get_attribute_reply;
@@ -953,6 +1044,8 @@ static void xv_dispose (vo_driver_t *this_gen) {
 
   _x_alphablend_free(&this->alphablend_extra_data);
 
+  cm_close (this);
+
   free (this);
 }
 
@@ -1109,6 +1202,16 @@ static void xv_update_xv_pitch_alignment(void *this_gen, xine_cfg_entry_t *entry
   this->use_pitch_alignment = entry->num_value;
 }
 
+static void xv_fullrange_cb_config (void *this_gen, xine_cfg_entry_t *entry) {
+  xv_driver_t *this = (xv_driver_t *)this_gen;
+  this->fullrange_mode = entry->num_value;
+  if (this->fullrange_mode)
+    this->capabilities |= VO_CAP_FULLRANGE;
+  else
+    this->capabilities &= ~VO_CAP_FULLRANGE;
+  this->cm_active = 0;
+}
+
 static xcb_xv_port_t xv_open_port (xv_driver_t *this, xcb_xv_port_t port) {
   xcb_xv_grab_port_cookie_t grab_port_cookie;
   xcb_xv_grab_port_reply_t *grab_port_reply;
@@ -1279,6 +1382,7 @@ static vo_driver_t *open_plugin(video_driver_class_t *class_gen, const void *vis
   this->xoverlay                = NULL;
   this->ovl_changed             = 0;
   this->xine                    = class->xine;
+  this->cm_atom                 = XCB_NONE;
 
   this->vo_driver.get_capabilities     = xv_get_capabilities;
   this->vo_driver.alloc_frame          = xv_alloc_frame;
@@ -1303,6 +1407,7 @@ static vo_driver_t *open_plugin(video_driver_class_t *class_gen, const void *vis
     this->props[i].min   = 0;
     this->props[i].max   = 0;
     this->props[i].atom  = XCB_NONE;
+    this->props[i].defer = 0;
     this->props[i].entry = NULL;
     this->props[i].this  = this;
   }
@@ -1312,6 +1417,8 @@ static vo_driver_t *open_plugin(video_driver_class_t *class_gen, const void *vis
   this->props[VO_PROP_ZOOM_X].value          = 100;
   this->props[VO_PROP_ZOOM_Y].value          = 100;
 
+  cm_init (this);
+
   /*
    * check this adaptor's capabilities
    */
@@ -1358,6 +1465,30 @@ static vo_driver_t *open_plugin(video_driver_class_t *class_gen, const void *vis
 	  xv_check_capability (this, VO_PROP_GAMMA, attribute_it.data,
 			       adaptor_it.data->base_id,
 			       NULL, NULL, NULL);
+	} else if(!strcmp(name, "XV_ITURBT_709")) {
+	  xcb_xv_get_port_attribute_cookie_t get_attribute_cookie;
+	  xcb_xv_get_port_attribute_reply_t *get_attribute_reply;
+	  xcb_intern_atom_cookie_t atom_cookie;
+	  xcb_intern_atom_reply_t *atom_reply;
+	  pthread_mutex_lock(&this->main_mutex);
+	  atom_cookie = xcb_intern_atom (this->connection, 0, 13, name);
+	  atom_reply = xcb_intern_atom_reply (this->connection, atom_cookie, NULL);
+	  if (atom_reply) {
+	    this->cm_atom = atom_reply->atom;
+	    free (atom_reply);
+	  }
+	  if (this->cm_atom != XCB_NONE) {
+	    get_attribute_cookie = xcb_xv_get_port_attribute (this->connection,
+	      this->xv_port, this->cm_atom);
+	    get_attribute_reply = xcb_xv_get_port_attribute_reply (this->connection,
+	      get_attribute_cookie, NULL);
+	    if (get_attribute_reply) {
+	      this->cm_active = get_attribute_reply->value ? 2 : 10;
+	      free(get_attribute_reply);
+	      this->capabilities |= VO_CAP_COLOR_MATRIX;
+	    }
+	  }
+	  pthread_mutex_unlock(&this->main_mutex);
 	} else if(!strcmp(name, "XV_COLORKEY")) {
 	  this->capabilities |= VO_CAP_COLORKEY;
 	  xv_check_capability (this, VO_PROP_COLORKEY, attribute_it.data,
@@ -1412,6 +1543,33 @@ static vo_driver_t *open_plugin(video_driver_class_t *class_gen, const void *vis
     xprintf(this->xine, XINE_VERBOSITY_DEBUG, LOG_MODULE ": no port attributes defined.\n");
   free(query_adaptors_reply);
 
+  this->props[VO_PROP_BRIGHTNESS].defer = 1;
+  this->props[VO_PROP_CONTRAST].defer   = 1;
+  this->props[VO_PROP_SATURATION].defer = 1;
+  if ((this->props[VO_PROP_BRIGHTNESS].atom != XCB_NONE) &&
+    (this->props[VO_PROP_CONTRAST].atom != XCB_NONE)) {
+    static const char * const xv_fullrange_conf_labels[] = {"Off", "Normal", NULL};
+    this->fullrange_mode = this->xine->config->register_enum (
+      this->xine->config,
+      "video.output.xv_fullrange",
+      0,
+      (char **)xv_fullrange_conf_labels,
+      _("Fullrange color emulation"),
+      _("Emulate fullrange color by modifying brightness/contrast/saturation settings.\n\n"
+        "Off:    Let decoders convert such video.\n"
+        "        Works with all graphics cards, but needs a bit more CPU power.\n\n"
+        "Normal: Fast and better quality. Should work at least with some newer cards\n"
+        "        (GeForce 7+, 210+).\n\n"),
+      10,
+      xv_fullrange_cb_config,
+      this
+    );
+    if (this->fullrange_mode)
+      this->capabilities |= VO_CAP_FULLRANGE;
+  } else {
+    this->fullrange_mode = 0;
+  }
+
   /*
    * check supported image formats
    */
