@@ -61,9 +61,9 @@
 #include <dlfcn.h>
 #include <pthread.h>
 
-#include "xine_internal.h"
-#include "buffer.h"
-#include "xineutils.h"
+#include <xine/xine_internal.h>
+#include <xine/buffer.h>
+#include <xine/xineutils.h>
 #include "xine_private.h"
 
 #define QLEN 5    /* maximum connection queue length */
@@ -113,9 +113,10 @@ static int sock_check_opened(int socket) {
 /*
  * Write to socket.
  */
-static int sock_data_write(xine_t *xine, int socket, char *buf, int len) {
+static int sock_data_write(xine_t *xine, int socket, void *buf_gen, int len) {
   ssize_t  size;
   int      wlen = 0;
+  uint8_t *buf = buf_gen;
 
   if((socket < 0) || (buf == NULL))
     return -1;
@@ -140,7 +141,7 @@ static int sock_data_write(xine_t *xine, int socket, char *buf, int len) {
 }
 
 static int XINE_FORMAT_PRINTF(3, 4)
-sock_string_write(xine_t *xine, int socket, char *msg, ...) {
+sock_string_write(xine_t *xine, int socket, const char *msg, ...) {
   char     buf[_BUFSIZ];
   va_list  args;
 
@@ -159,7 +160,7 @@ sock_string_write(xine_t *xine, int socket, char *msg, ...) {
  * this is the most important broadcaster function.
  * it sends data to every connected client (slaves).
  */
-static void broadcaster_data_write(broadcaster_t *this, char *buf, int len) {
+static void broadcaster_data_write(broadcaster_t *this, void *buf, int len) {
   xine_list_iterator_t ite;
 
   ite = xine_list_front (this->connections);
@@ -182,7 +183,7 @@ static void broadcaster_data_write(broadcaster_t *this, char *buf, int len) {
 }
 
 static void XINE_FORMAT_PRINTF(2, 3)
-broadcaster_string_write(broadcaster_t *this, char *msg, ...) {
+broadcaster_string_write(broadcaster_t *this, const char *msg, ...) {
   char     buf[_BUFSIZ];
   va_list  args;
 
@@ -254,7 +255,7 @@ static void *manager_loop (void *this_gen) {
 /*
  * receive xine buffers and send them through the broadcaster
  */
-static void send_buf (broadcaster_t *this, char *from, buf_element_t *buf) {
+static void send_buf (broadcaster_t *this, const char *from, buf_element_t *buf) {
   int i;
 
   /* ignore END buffers since they would stop the slavery */
